(** This module provides common functions.

    Last changed: 20.11.2015 *)

<* O2EXTENSIONS + *>
MODULE mCommonFunctions;

IMPORT
  Math:=mMath, SU:=msysUtils, COMPILER, Out:=msysTextOut;

CONST
  ModuleName = COMPILER.EQUATION('module');

TYPE
  Float * = Math.Float;
  Complex = Float;

(* =============== Functions of one real variable =========================== *)
TYPE
  Float1 * = RECORD (** Common function of one argument *)
  END;

PROCEDURE (VAR f:Float1) f * (x:Float): Complex;
  BEGIN
    SU.RAISEError(ModuleName+'.Float1.f: abstract method');
    RETURN x;
  END f;

TYPE
  Simple1 * = PROCEDURE (x:Float): Complex; (** Simple function of one argument *)
  SimpleFloat1 = RECORD (Float1)
    sf:Simple1;
  END;

PROCEDURE (VAR f:SimpleFloat1) f (x:Float): Complex;
  BEGIN
    RETURN f.sf(x)
  END f;

PROCEDURE FromSimple1 * (f:Simple1): POINTER TO Float1;
  (** Converts a simple function to the common one *)
  VAR
    res:POINTER TO SimpleFloat1;
  BEGIN
    ASSERT(f#NIL);

    NEW(res);
    res.sf:=f;
    RETURN res
  END FromSimple1;

TYPE
  ConstFloat1 = RECORD (Float1)
    val:Complex;
  END;

<* PUSH *>
<* WOFF301 + *>
PROCEDURE (VAR f:ConstFloat1) f (x:Float): Complex;
  BEGIN
    RETURN f.val;
  END f;
<* POP *>

PROCEDURE Constant1 * (val:Complex): POINTER TO Float1;
  (** Returns function, which returns just the specified constant *)
  VAR
    res:POINTER TO ConstFloat1;
  BEGIN
    NEW(res);
    res.val:=val;
    RETURN res
  END Constant1;

PROCEDURE Export1 * (VAR f:Float1; MinX,MaxX,dx:Float; FileName-:ARRAY OF CHAR;
                    factor:Complex);
  (** Writes values of the specified function multiplied by the specified factor
      in the specified range to the specified file, in text-columns format.
      If the file already exists, it is overwritten. *)
  VAR
    x:Float;
    fm:Out.Formatter;
  PROCEDURE SaveLine ();
    CONST
      Tab = Out.Tab;
    BEGIN
      Out.Real(fm,x,0);
      Out.String(fm,Tab,0);
      Out.Real(fm,f.f(x)*factor,0);
      Out.Ln(fm);
    END SaveLine;
  VAR
    tmp:Float;
  BEGIN
    ASSERT(dx#0);

    Out.OpenFile(fm,FileName,Out.Rewrite);

    dx:=ABS(dx);
    IF MinX>MaxX THEN
      tmp:=MinX;
      MinX:=MaxX;
      MaxX:=tmp;
      dx:=-dx;
    END;

    x:=MinX;
    REPEAT
      SaveLine;
      x:=x+dx
    UNTIL x>=MaxX;
    x:=MaxX;
    SaveLine;

    Out.Close(fm);
  END Export1;

PROCEDURE ExportSimple1 * (f:Simple1; MinX,MaxX,dx:Float; FileName-:ARRAY OF CHAR;
                           factor:Complex);
  (** Writes values of the specified function multiplied by the specified factor
      in the specified range to the specified file, in text-columns format.
      If the file already exists, it is overwritten. *)
  BEGIN
    ASSERT(f#NIL);

    Export1(FromSimple1(f)^,MinX,MaxX,dx,FileName,factor);
  END ExportSimple1;

(* =============== Functions of two real variables ========================== *)

TYPE
  Float2 * = RECORD (** Common function of two arguments *)
  END;

PROCEDURE (VAR f:Float2) f * (x,y:Float): Complex;
  BEGIN
    SU.RAISEError(ModuleName+'.Float2.f: abstract method');
    RETURN x-y;
  END f;

TYPE
  Simple2 * = PROCEDURE (x,y:Float): Complex; (** Simple function of two arguments *)
  SimpleFloat2 = RECORD (Float2)
    sf:Simple2;
  END;

PROCEDURE (VAR f:SimpleFloat2) f (x,y:Float): Complex;
  BEGIN
    RETURN f.sf(x,y)
  END f;

PROCEDURE FromSimple2 * (f:Simple2): POINTER TO Float2;
  (** Converts a simple function to the common one *)
  VAR
    res:POINTER TO SimpleFloat2;
  BEGIN
    ASSERT(f#NIL);

    NEW(res);
    res.sf:=f;
    RETURN res
  END FromSimple2;

TYPE
  ConstFloat2 = RECORD (Float2)
    val:Complex;
  END;

<* PUSH *>
<* WOFF301 + *>
PROCEDURE (VAR f:ConstFloat2) f (x,y:Float): Complex;
  BEGIN
    RETURN f.val;
  END f;
<* POP *>

PROCEDURE Constant2 * (val:Complex): POINTER TO Float2;
  (** Returns function, which returns just the specified constant *)
  VAR
    res:POINTER TO ConstFloat2;
  BEGIN
    NEW(res);
    res.val:=val;
    RETURN res
  END Constant2;

(* =============== Functions of two integer variables ======================= *)

TYPE
  Int2 * = RECORD (** Common function of two integer arguments *)
  END;

PROCEDURE (VAR f:Int2) f * (x,y:LONGINT): Complex;
  BEGIN
    SU.RAISEError(ModuleName+'.Int2.f: abstract method');
    RETURN x-y;
  END f;

TYPE
  ConstInt2 = RECORD (Int2)
    val:Complex;
  END;

<* PUSH *>
<* WOFF301 + *>
PROCEDURE (VAR f:ConstInt2) f (x,y:LONGINT): Complex;
  BEGIN
    RETURN f.val;
  END f;
<* POP *>

PROCEDURE ConstantInt2 * (val:Complex): POINTER TO Int2;
  (** Returns function, which returns just the specified constant *)
  VAR
    res:POINTER TO ConstInt2;
  BEGIN
    NEW(res);
    res.val:=val;
    RETURN res
  END ConstantInt2;

END mCommonFunctions.