(** This module provides common functions.

    Last changed: 20.11.2015 *)

<* O2EXTENSIONS + *>
MODULE mCommonFunctionsComplex;

IMPORT
  Math:=mMath, MathComplex:=mMathComplex, SU:=msysUtils, COMPILER;

CONST
  ModuleName = COMPILER.EQUATION('module');

TYPE
  Float * = Math.Float;
  Complex * = MathComplex.Complex;

(* =============== Functions of one real variable =========================== *)
TYPE
  Float1 * = RECORD (** Common function of one argument *)
  END;

PROCEDURE (VAR f:Float1) f * (x:Float): Complex;
  BEGIN
    SU.RAISEError(ModuleName+'.Float1.f: abstract method');
    RETURN x;
  END f;

TYPE
  Simple1 * = PROCEDURE (x:Float): Complex; (** Simple function of one argument *)
  SimpleFloat1 = RECORD (Float1)
    sf:Simple1;
  END;

PROCEDURE (VAR f:SimpleFloat1) f (x:Float): Complex;
  BEGIN
    RETURN f.sf(x)
  END f;

PROCEDURE FromSimple1 * (f:Simple1): POINTER TO Float1;
  (** Converts a simple function to the common one *)
  VAR
    res:POINTER TO SimpleFloat1;
  BEGIN
    ASSERT(f#NIL);

    NEW(res);
    res.sf:=f;
    RETURN res
  END FromSimple1;

TYPE
  ConstFloat1 = RECORD (Float1)
    val:Complex;
  END;

<* PUSH *>
<* WOFF301 + *>
PROCEDURE (VAR f:ConstFloat1) f (x:Float): Complex;
  BEGIN
    RETURN f.val;
  END f;
<* POP *>

PROCEDURE Constant1 * (val:Complex): POINTER TO Float1;
  (** Returns function, which returns just the specified constant *)
  VAR
    res:POINTER TO ConstFloat1;
  BEGIN
    NEW(res);
    res.val:=val;
    RETURN res
  END Constant1;

(* =============== Functions of two real variables ========================== *)

TYPE
  Float2 * = RECORD (** Common function of two arguments *)
  END;

PROCEDURE (VAR f:Float2) f * (x,y:Float): Complex;
  BEGIN
    SU.RAISEError(ModuleName+'.Float2.f: abstract method');
    RETURN x-y;
  END f;

TYPE
  Simple2 * = PROCEDURE (x,y:Float): Complex; (** Simple function of two arguments *)
  SimpleFloat2 = RECORD (Float2)
    sf:Simple2;
  END;

PROCEDURE (VAR f:SimpleFloat2) f (x,y:Float): Complex;
  BEGIN
    RETURN f.sf(x,y)
  END f;

PROCEDURE FromSimple2 * (f:Simple2): POINTER TO Float2;
  (** Converts a simple function to the common one *)
  VAR
    res:POINTER TO SimpleFloat2;
  BEGIN
    ASSERT(f#NIL);

    NEW(res);
    res.sf:=f;
    RETURN res
  END FromSimple2;

TYPE
  ConstFloat2 = RECORD (Float2)
    val:Complex;
  END;

<* PUSH *>
<* WOFF301 + *>
PROCEDURE (VAR f:ConstFloat2) f (x,y:Float): Complex;
  BEGIN
    RETURN f.val;
  END f;
<* POP *>

PROCEDURE Constant2 * (val:Complex): POINTER TO Float2;
  (** Returns function, which returns just the specified constant *)
  VAR
    res:POINTER TO ConstFloat2;
  BEGIN
    NEW(res);
    res.val:=val;
    RETURN res
  END Constant2;

(* =============== Functions of two integer variables ======================= *)

TYPE
  Int2 * = RECORD (** Common function of two integer arguments *)
  END;

PROCEDURE (VAR f:Int2) f * (x,y:LONGINT): Complex;
  BEGIN
    SU.RAISEError(ModuleName+'.Int2.f: abstract method');
    RETURN x-y;
  END f;

TYPE
  ConstInt2 = RECORD (Int2)
    val:Complex;
  END;

<* PUSH *>
<* WOFF301 + *>
PROCEDURE (VAR f:ConstInt2) f (x,y:LONGINT): Complex;
  BEGIN
    RETURN f.val;
  END f;
<* POP *>

PROCEDURE ConstantInt2 * (val:Complex): POINTER TO Int2;
  (** Returns function, which returns just the specified constant *)
  VAR
    res:POINTER TO ConstInt2;
  BEGIN
    NEW(res);
    res.val:=val;
    RETURN res
  END ConstantInt2;

END mCommonFunctionsComplex.