(** This module provides some low level procedures concerning the
    mathematical processing.

    Last change: 16.11.2007 *)
<* O2EXTENSIONS + *>
MODULE mLowMath;

IMPORT
  mMath;

TYPE
  Float * = mMath.Float;

PROCEDURE SinCos * (theta:Float; VAR sin,cos:Float);
  (** Returns sin and cos of theta (theta is in radians) *)
  BEGIN
    ASM
      fld       theta
      fsincos
      mov       EAX,dword ptr [cos]
      mov       EDX,dword ptr [sin]
<* IF Float="REAL" THEN *>
      fstp      dword ptr [EAX]
      fstp      dword ptr [EDX]
<* ELSIF Float="LONGREAL" THEN *>
      fstp      qword ptr [EAX]
      fstp      qword ptr [EDX]
<* ELSE *>
      fstp      tbyte ptr [EAX]
      fstp      tbyte ptr [EDX]
<* END *>
    END;
  END SinCos;

PROCEDURE Set8087CW (value:INTEGER);
  BEGIN
    ASM
      FLDCW     value
    END;
  END Set8087CW;

PROCEDURE Get8087CW ():INTEGER;
  VAR
    result:INTEGER;
  BEGIN
    ASM
      FSTCW     result
    END;
    RETURN result
  END Get8087CW;

PROCEDURE SetREALPrecision * ();
  (** Sets precision of floating point operations corresponding to REAL type. *)
  BEGIN
    Set8087CW(Get8087CW() AND 0FCFFh)
  END SetREALPrecision;

PROCEDURE SetLONGREALPrecision * ();
  (** Sets precision of floating point operations corresponding to LONGREAL type. *)
  BEGIN
    Set8087CW((Get8087CW() AND 0FCFFh) OR 0200h)
  END SetLONGREALPrecision;

PROCEDURE SetLONGLONGREALPrecision * ();
  (** Sets precision of floating point operations corresponding to LONGLONGREAL type. *)
  BEGIN
    Set8087CW(Get8087CW() OR 0300h)
  END SetLONGLONGREALPrecision;

PROCEDURE SetFloatPrecision * ();
  (** Sets precision of floating point operations corresponding to LONGLONGREAL type. *)
  BEGIN
    <* IF Float="REAL" THEN *>          SetREALPrecision
    <* ELSIF Float="LONGREAL" THEN *>   SetLONGREALPrecision
    <* ELSE *>                          SetLONGLONGREALPrecision
    <* END *>
  END SetFloatPrecision;

END mLowMath.