(** This module defines the basic numerical type Complex and some constants and
    procedures for it. The type complex corresponds to the type Float.

    Last change: 12.07.2014 *)
<* O2NUMEXT + *>
MODULE mMathComplex;

IMPORT
  Math:=mMath;

<* IF Float="REAL" THEN *>

IMPORT
  MathC:=ComplexMath;

TYPE
  Complex * = COMPLEX;

<* ELSIF Float="LONGREAL" THEN *>

IMPORT
  MathC:=LongComplexMath;

TYPE
  Complex * = LONGCOMPLEX;

<* ELSIF Float="LONGLONGREAL" THEN *>
  'ERROR: not supported for LONGLONGREAL
<* ELSE *>
  'ERROR: equation Float is not defined
<* END *>

CONST
  pi * = Math.pi;
  i * = MathC.i;
  one * = MathC.one;
  zero * = MathC.zero;

TYPE
  Float * = Math.Float;
  Array * = ARRAY OF Complex;
  ArrayPtr * = POINTER TO Array;
  Array2 * = ARRAY OF ARRAY OF Complex;
  Array2Ptr * = POINTER TO Array2;

PROCEDURE FloatSqr (z:Float): Float;
  BEGIN
    RETURN z*z
  END FloatSqr;

PROCEDURE SqrAbs * (z:Complex): Float;
  (** Returns ABS(z)^2 *)
  BEGIN
    RETURN FloatSqr(RE(z)) + FloatSqr(IM(z))
  END SqrAbs;

PROCEDURE sqr * (z:Complex): Complex;
  (** Returns z*z *)
  BEGIN
    RETURN z*z
  END sqr;

(** Following are procedures similar to procedures from
    ComplexMath and LongComplexMath modules *)

PROCEDURE abs * (z:Complex): Float;
  BEGIN
    RETURN MathC.abs(z)
  END abs;

PROCEDURE arg * (z:Complex): Float;
  BEGIN
    RETURN MathC.arg(z)
  END arg;

PROCEDURE conj * (z:Complex): Complex;
  BEGIN
    RETURN MathC.conj(z)
  END conj;

PROCEDURE power * (base:Complex; exponent:Float): Complex;
  BEGIN
    RETURN MathC.power(base,exponent)
  END power;

PROCEDURE sqrt * (z:Complex): Complex;
  BEGIN
    RETURN MathC.sqrt(z)
  END sqrt;

PROCEDURE exp * (z:Complex): Complex;
  BEGIN
    RETURN MathC.exp(z)
  END exp;

PROCEDURE ln * (z:Complex): Complex;
  BEGIN
    RETURN MathC.ln(z)
  END ln;

PROCEDURE sin * (z:Complex): Complex;
  BEGIN
    RETURN MathC.sin(z)
  END sin;

PROCEDURE cos * (z:Complex): Complex;
  BEGIN
    RETURN MathC.cos(z)
  END cos;

PROCEDURE tan * (z:Complex): Complex;
  BEGIN
    RETURN MathC.tan(z)
  END tan;

PROCEDURE arcsin * (z:Complex): Complex;
  BEGIN
    RETURN MathC.arcsin(z)
  END arcsin;

PROCEDURE arccos * (z:Complex): Complex;
  BEGIN
    RETURN MathC.arccos(z)
  END arccos;

PROCEDURE arctan * (z:Complex): Complex;
  BEGIN
    RETURN MathC.arctan(z)
  END arctan;

PROCEDURE polarToComplex * (abs,arg:Float): Complex;
  BEGIN
    RETURN MathC.polarToComplex(abs,arg)
  END polarToComplex;

PROCEDURE scalarMult * (scalar:Float; z:Complex): Complex;
  BEGIN
    RETURN MathC.scalarMult(scalar,z);
  END scalarMult;

PROCEDURE IsCMathException * (): BOOLEAN;
  BEGIN
    RETURN MathC.IsCMathException()
  END IsCMathException;

END mMathComplex.