(** Procedures of this module are similar to msysTextOut module, but they write
    data to the standard output.

    Last change: 27.05.2015 *)
<* O2EXTENSIONS + *>
MODULE mhostStdOut;

IMPORT
  ByteStream:=msysStreamOfBytes, Out:=msysTextOut, SYS:=SYSTEM, Win:=Windows;

CONST
  Eoln * = Out.Eoln; (** End of line mark *)
  Tab * = Out.Tab;   (** Tab *)

TYPE
  BYTE = SYS.BYTE;

  Writer = POINTER TO RECORD (ByteStream.WriterDesc)
    h:Win.HANDLE;
  END;

<* PUSH *>
<* WOFF304 + *>
PROCEDURE (w:Writer) Write (data-:ARRAY OF BYTE; count:LONGINT): LONGINT;
  VAR
    CountWritten:Win.DWORD;
  BEGIN
    ASSERT(count>=0);
    ASSERT(count<=LEN(data));

    Win.WriteFile(w.h,data,count,CountWritten,NIL);
    RETURN CountWritten;
  END Write;
<* POP *>

VAR
  f * : Out.Formatter; (** The formatter connected to the standard output *)

(** For description of procedures see TextOut module *)

PROCEDURE Char * (c:CHAR);
  BEGIN
    Out.Char(f,c)
  END Char;

PROCEDURE String * (str-:ARRAY OF CHAR; MinLength:LONGINT);
  BEGIN
    Out.String(f,str,MinLength);
  END String;

PROCEDURE DString * (str-:ARRAY OF CHAR; MinLength:LONGINT);
  BEGIN
    Out.DString(f,str,MinLength);
  END DString;

PROCEDURE Ln * ();
  BEGIN
    Out.Ln(f);
  END Ln;

PROCEDURE Int * (value:LONGLONGINT; MinLength:LONGINT);
  BEGIN
    Out.Int(f,value,MinLength);
  END Int;

PROCEDURE Real * (value:LONGLONGREAL; MinLength:LONGINT);
  BEGIN
    Out.Real(f,value,MinLength);
  END Real;

PROCEDURE RealEx * (value:LONGLONGREAL; digits,MinLength:LONGINT);
  BEGIN
    Out.RealEx(f,value,digits,MinLength);
  END RealEx;

PROCEDURE RealFixed * (value:LONGLONGREAL; digits,MinLength:LONGINT);
  BEGIN
    Out.RealFixed(f,value,digits,MinLength);
  END RealFixed;

PROCEDURE Bool * (value:BOOLEAN; MinLength:LONGINT);
  BEGIN
    Out.Bool(f,value,MinLength);
  END Bool;

PROCEDURE Set * (s:SET; MinLength:LONGINT);
  BEGIN
    Out.Set(f,s,MinLength);
  END Set;

PROCEDURE Init;
  VAR
    w:Writer;
  BEGIN
    NEW(w);
    w.h:=Win.GetStdHandle(Win.STD_OUTPUT_HANDLE);
    ASSERT(w.h#Win.INVALID_HANDLE_VALUE); (* not failed *)
    ASSERT(w.h#NIL); (* the standard channel exists *)
    Out.ConnectTo(f,w);
  END Init;

BEGIN
  Init
END mhostStdOut.