(** This file contains file reader and writer.

    Last changed: 30.03.2015 *)

<* O2EXTENSIONS + *>
MODULE mhostStreamOfFile;

IMPORT
  ByteStream:=msysStreamOfBytes, SYS:=SYSTEM, Win:=Windows, RTS:=oberonRTS,
  SU:=msysUtils, COMPILER, DStr:=msysDynStr;

TYPE
  BYTE = SYS.BYTE;
  DWORD = Win.DWORD;

CONST
  BufLen = 1024;

  ModuleName = COMPILER.EQUATION('module');

TYPE
  Reader * = POINTER TO RECORD (ByteStream.ReaderDesc)
    (** the reader *)
    h:Win.HANDLE;
    buf:ARRAY BufLen OF BYTE;
    ind,len:DWORD;
  END;

<* PUSH *>
<* WOFF304 + *>
PROCEDURE (r:Reader) Read * (VAR data:ARRAY OF BYTE; count:LONGINT): LONGINT;
  VAR
    d,CountRead,CountD:DWORD;
  BEGIN
    ASSERT(count>=0);
    ASSERT(count<=LEN(data));

    CountD:=count;
    CountRead:=0;
    d:=r.ind;
    WHILE (d<r.len) & (CountRead<CountD) DO
      data[CountRead]:=r.buf[d];
      CountRead:=CountRead+1;
      d:=d+1;
    END;
    r.ind:=r.ind+CountRead;
    IF CountD>CountRead THEN
      Win.ReadFile(r.h,SYS.M2ADR(data[CountRead]),CountD-CountRead,d,NIL);
      CountRead:=CountRead+d;
      IF CountRead=CountD THEN
        Win.ReadFile(r.h,r.buf,BufLen,r.len,NIL);
        r.ind:=0;
      END;
    END;

    RETURN CountRead;
  END Read;
<* POP *>

PROCEDURE ReaderFinalizer (x:SYS.ADDRESS);
  VAR
    r:Reader;
  BEGIN
    r:=SYS.VAL(Reader,x);
    Win.CloseHandle(r.h);
  END ReaderFinalizer;

PROCEDURE NewReader * (FileName-:ARRAY OF CHAR): Reader;
  (** Opens the file for read and returns the reader.
      The file is closed at finalizer of the reader *)
  VAR
    r:Reader;
    h:Win.HANDLE;
  BEGIN
    h:=Win.CreateFile(FileName,Win.GENERIC_READ,Win.FILE_SHARE_READ,NIL,
                      Win.OPEN_EXISTING,Win.FILE_FLAG_SEQUENTIAL_SCAN,NIL);
    IF h=Win.INVALID_HANDLE_VALUE THEN
      SU.RAISEError(DStr.Concat3(ModuleName+'.NewReader: can not open file "',FileName,'"')^);
      RETURN NIL
    ELSE
      NEW(r);
      r.h:=h;
      r.ind:=0;
      r.len:=0;
      RTS.InstallFinalizer(ReaderFinalizer,r);
      RETURN r
    END;
  END NewReader;

PROCEDURE CloseReader * (VAR r:Reader);
  (** NILs the r and tries to call r.Finalizer *)
  BEGIN
    r:=NIL;
    RTS.Collect();
  END CloseReader;

TYPE
  Writer * = POINTER TO RECORD (ByteStream.WriterDesc)
    (** The writer *)
    h:Win.HANDLE;
    buf:ARRAY BufLen OF BYTE;
    len:DWORD;
  END;

PROCEDURE DataToBuf (w:Writer; data-:ARRAY OF BYTE; count:DWORD);
  VAR
    i,len:DWORD;
  BEGIN
    ASSERT(w.len+count<BufLen);
    len:=w.len;
    i:=0;
    WHILE i<count DO
      w.buf[len+i]:=data[i];
      i:=i+1;
    END;
    w.len:=w.len+count;
  END DataToBuf;

<* PUSH *>
<* WOFF304 + *>
PROCEDURE BufToFile (w:Writer);
  VAR
    d:DWORD;
  BEGIN
    Win.WriteFile(w.h,w.buf,w.len,d,NIL);
    ASSERT(d=w.len);
    w.len:=0;
  END BufToFile;

PROCEDURE (w:Writer) Write * (data-:ARRAY OF BYTE; count:LONGINT): LONGINT;
  VAR
    CountD,d:DWORD;
  BEGIN
    ASSERT(count>=0);
    ASSERT(count<=LEN(data));

    CountD:=count;
    IF w.len+CountD<BufLen THEN
      DataToBuf(w,data,CountD);
    ELSE
      BufToFile(w);
      IF count<BufLen THEN
        DataToBuf(w,data,CountD)
      ELSE
        Win.WriteFile(w.h,data,count,d,NIL);
        ASSERT(d=CountD);
      END;
    END;
    RETURN count;
  END Write;
<* POP *>

PROCEDURE WriterFinalizer (x:SYS.ADDRESS);
  VAR
    w:Writer;
  BEGIN
    w:=SYS.VAL(Writer,x);
    BufToFile(w);
    Win.CloseHandle(w.h);
  END WriterFinalizer;

PROCEDURE OpenForWrite (FileName-:ARRAY OF CHAR;
                        disposition:Win.CREATION_DISTRIBUTION): Writer;
  VAR
    w:Writer;
    h:Win.HANDLE;
  BEGIN
    h:=Win.CreateFile(FileName,Win.GENERIC_WRITE,Win.FILE_SHARE_READ,NIL,
                      disposition,Win.FILE_FLAG_SEQUENTIAL_SCAN,NIL);
    IF h=Win.INVALID_HANDLE_VALUE THEN
      RETURN NIL
    ELSE
      NEW(w);
      w.h:=h;
      RTS.InstallFinalizer(WriterFinalizer,w);
      RETURN w
    END;
  END OpenForWrite;

PROCEDURE RewriteFile (FileName-:ARRAY OF CHAR): Writer;
  BEGIN
    RETURN OpenForWrite(FileName,Win.CREATE_ALWAYS);
  END RewriteFile;

PROCEDURE SetLastPos (h:Win.HANDLE);
  CONST
    INVALID_SET_FILE_POINTER = 0FFFFFFFFh;
  VAR
    hi:Win.LONG;
    res:DWORD;
  BEGIN
    hi:=0;
    res:=Win.SetFilePointer(h,0,hi,Win.FILE_END);
    ASSERT((res#INVALID_SET_FILE_POINTER) OR (Win.GetLastError()=Win.NO_ERROR));
  END SetLastPos;

PROCEDURE AppendFile (FileName-:ARRAY OF CHAR): Writer;
  VAR
    w:Writer;
  BEGIN
    w:=OpenForWrite(FileName,Win.OPEN_ALWAYS);
    IF w#NIL THEN
      SetLastPos(w.h);
    END;
    RETURN w;
  END AppendFile;

TYPE
  Mode * = (Rewrite,Append);
  (** Mode of the file opening *)

PROCEDURE NewWriter * (FileName-:ARRAY OF CHAR; mode:Mode): Writer;
  (** Rewrites or appends the file. If the file did not exist it is created
      with any mode parameter. Returns the writer.
      The file is closed at finalizer of the writer *)
  VAR
    w:Writer;
  BEGIN
    CASE mode OF
      Rewrite:
        w:=RewriteFile(FileName)
    | Append:
        w:=AppendFile(FileName)
    END;
    IF w=NIL THEN
      SU.RAISEError(DStr.Concat3(ModuleName+'.NewWriter: can not open file "',FileName,'"')^);
    ELSE
      w.len:=0;
      RETURN w;
    END;
  END NewWriter;

PROCEDURE CloseWriter * (VAR w:Writer);
  (** NILs the w and calls r.Finalizer, if possible *)
  BEGIN
    w:=NIL;
    RTS.Collect();
  END CloseWriter;

PROCEDURE Flush * (w:Writer);
  (** Flushes the writer *)
  VAR
    b:Win.BOOL;
  BEGIN
    ASSERT(w#NIL);

    BufToFile(w);
    b:=Win.FlushFileBuffers(w.h);
    ASSERT(b);
  END Flush;

END mhostStreamOfFile.