(** This module contains an abstract definitions of sequential byte reader
    and writer.

    Last change: 26.03.2015 *)

<* O2EXTENSIONS + *>
MODULE msysStreamOfBytes;

IMPORT
  SYS:=SYSTEM, SU:=msysUtils, COMPILER;

CONST
  ModuleName = COMPILER.EQUATION('Module');

TYPE
  BYTE = SYS.BYTE;

  ReaderDesc * = RECORD
  END;
  Reader * = POINTER TO ReaderDesc;
  (** Abstract reader of bytes *)

PROCEDURE (r:Reader) Read * (VAR data:ARRAY OF BYTE; count:LONGINT): LONGINT;
  (** Should read count butes from the stream to data. Should return
      number of actually read bytes. *)
  BEGIN
    SU.RAISEError(ModuleName+'.Reader.Read: abstract method');
    RETURN LEN(data)+count
  END Read;

TYPE
  WriterDesc * = RECORD
  END;
  Writer * = POINTER TO WriterDesc;
  (** Abstract writer of bytes. *)

PROCEDURE (w:Writer) Write * (data-:ARRAY OF BYTE; count:LONGINT): LONGINT;
  (** Should write count butes from data to the stream. Should return
      number of actually written bytes. *)
  BEGIN
    SU.RAISEError(ModuleName+'.Writer.Write: abstract method');
    RETURN LEN(data)+count
  END Write;

PROCEDURE Copy * (source:Reader; dest:Writer; BufSize:LONGINT);
  (** Copies the source to the desc with specified internal buffer. *)
  VAR
    buf:POINTER TO ARRAY OF BYTE;
    CountRead,CountWritten:LONGINT;
  BEGIN
    ASSERT(source#NIL);
    ASSERT(dest#NIL);
    ASSERT(BufSize>0);

    NEW(buf,BufSize);
    REPEAT
      CountRead:=source.Read(buf^,BufSize);
      CountWritten:=dest.Write(buf^,CountRead);
    UNTIL (CountRead#BufSize) OR (CountWritten#CountRead)
  END Copy;

END msysStreamOfBytes.