(** This module contain different system routines collection.

    Last change: 26.03.2015 *)

<* O2EXTENSIONS + *>

MODULE msysUtils;

IMPORT
  DStr:=msysDynStr, SysClock, EXC:=EXCEPTIONS, FName:=FileName;

TYPE
  String = DStr.String;
  DateTime = SysClock.DateTime;

VAR
  ExcSource:EXC.ExceptionSource;

PROCEDURE RAISEError * (message-:ARRAY OF CHAR);
  (** Generates critical error with the message and terminates the program. *)
  BEGIN
    EXC.RAISE(ExcSource,0,message);
  END RAISEError;

PROCEDURE MsToStr * (ms:LONGINT; delimiter:CHAR): String;
  (** Converts milliseconds to string representstion: h_mm_ss_iii
      'iii' is milliseconds, '_' is delimiter. *)
  VAR
    mili,s,m:LONGINT;
    str:String;
  BEGIN
    mili:=ms MOD 1000;
    ms:=ms DIV 1000;
    s:=ms MOD 60;
    ms:=ms DIV 60;
    m:=ms MOD 60;
    ms:=ms DIV 60;

    str:=DStr.Concat4(DStr.FromInt(ms)^, delimiter,
                      DStr.FromIntEx(m,2,0)^, delimiter);
    str:=DStr.Concat4(str^, DStr.FromIntEx(s,2,0)^,
                      delimiter, DStr.FromIntEx(mili,3,0)^);

    RETURN str
  END MsToStr;

PROCEDURE DateStr * (VAR datetime:DateTime; delimiter:CHAR): String;
  (** Converts date part of datetime to string dd_mm_yyyy.
      '_' is delimiter. *)
  VAR
    s:String;
  BEGIN
    s:=DStr.Concat4(DStr.FromIntEx(VAL(LONGINT,datetime.day),2,0)^, delimiter,
                    DStr.FromIntEx(VAL(LONGINT,datetime.month),2,0)^, delimiter);
    s:=DStr.Concat2(s^,DStr.FromInt(VAL(LONGINT,datetime.year))^);

    RETURN s
  END DateStr;

PROCEDURE TimeStr * (VAR datetime:DateTime; delimiter:CHAR): String;
  (** Converts time part of datetime to string hh_mm_ss_iii,
      'iii' is milliseconds, '_' is delimiter. *)
  VAR
    s:String;
  BEGIN
    s:=DStr.Concat4(DStr.FromInt(VAL(LONGINT,datetime.hour))^, delimiter,
                    DStr.FromIntEx(VAL(LONGINT,datetime.minute),2,0)^, delimiter);
    s:=DStr.Concat4(s^, DStr.FromIntEx(VAL(LONGINT,datetime.second),2,0)^,
                    delimiter, DStr.FromIntEx(VAL(LONGINT,datetime.fractions
                                                          *1000 DIV (SysClock.maxSecondParts+1)),3,0)^);
    RETURN s
  END TimeStr;

PROCEDURE ExtractPath * (FileName-:ARRAY OF CHAR): String;
  (** Returns path in the specified file name, or empty string if no path is specified *)
  VAR
    f:FName.Format;
  BEGIN
    FName.GetFormat(FileName,f);
    IF f.ok THEN
      RETURN DStr.Copy(FileName,f.dirPos,f.dirLen);
    ELSE
      RETURN DStr.Empty()
    END;
  END ExtractPath;

PROCEDURE ExtractName * (FileName-:ARRAY OF CHAR): String;
  (** Returns name in the specified file name, or empty string if no name is specified *)
  VAR
    f:FName.Format;
  BEGIN
    FName.GetFormat(FileName,f);
    IF f.ok THEN
      RETURN DStr.Copy(FileName,f.namePos,f.nameLen);
    ELSE
      RETURN DStr.Empty()
    END;
  END ExtractName;

PROCEDURE ExtractExt * (FileName-:ARRAY OF CHAR): String;
  (** Returns extension in the specified file name, or empty string if no extension is specified *)
  VAR
    f:FName.Format;
  BEGIN
    FName.GetFormat(FileName,f);
    IF f.ok THEN
      RETURN DStr.Copy(FileName,f.extPos,f.extLen);
    ELSE
      RETURN DStr.Empty()
    END;
  END ExtractExt;

PROCEDURE ExtractPathName * (FileName-:ARRAY OF CHAR): String;
  (** Returns path and name in the specified file name, or empty string if no
      path or name are specified *)
  VAR
    path,name,result:String;
  BEGIN
    path:=ExtractPath(FileName);
    name:=ExtractName(FileName);
    IF path^='' THEN
      result:=name
    ELSIF path[DStr.Length(path^)-1]='\' THEN
      result:=DStr.Concat2(path^,name^);
    ELSE
      result:=DStr.Concat3(path^,'\',name^);
    END;
    RETURN result
  END ExtractPathName;

PROCEDURE ExtractNameExt * (FileName-:ARRAY OF CHAR): String;
  (** Returns name and extension in the specified file name, or empty string if no
      name or extension are specified *)
  VAR
    name,ext,result:String;
  BEGIN
    name:=ExtractName(FileName);
    ext:=ExtractExt(FileName);
    IF name^='' THEN
      result:=ext
    ELSIF ext^='' THEN
      result:=name
    ELSE
      result:=DStr.Concat3(name^,'.',ext^);
    END;
    RETURN result;
  END ExtractNameExt;

BEGIN
  EXC.AllocateSource(ExcSource);
END msysUtils.