(** This module has the same procedures and structures as the module 'Semaphores',
    with a littile adaptation to the 'mthrSym' module.

    Last change: 27.03.2015 *)

<* WOFF301 + *>
<* WOFF311 + *>

MODULE mthrSemaphores;

IMPORT
  Thr:=mthrSym, TERMINATION;

IMPORT
  SEM:=Semaphores;

(*
IMPORT
  SEM:=SpinlockSemaphores; (* For spinlock semaphores *)
*)

TYPE
  SEMAPHORE * = SEM.SEMAPHORE;

PROCEDURE Create * (VAR s:SEMAPHORE; InitialCount:LONGINT);
  BEGIN
    ASSERT(InitialCount>=0);
    IF Thr.Count>1 THEN
      SEM.Create(s,InitialCount)
    END;
  END Create;

PROCEDURE Destroy * (VAR s:SEMAPHORE);
  BEGIN
    IF Thr.Count>1 THEN
      SEM.Destroy(s)
    END;
  END Destroy;

PROCEDURE Claim * (VAR s:SEMAPHORE);
  BEGIN
    IF (Thr.Count>1) & (~TERMINATION.HasHalted()) THEN
      SEM.Claim(s)
    END;
  END Claim;

PROCEDURE Release * (VAR s:SEMAPHORE);
  BEGIN
    IF (Thr.Count>1) & (~TERMINATION.HasHalted()) THEN
      SEM.Release(s)
    END;
  END Release;

PROCEDURE CondClaim * (s:SEMAPHORE): BOOLEAN;
  BEGIN
    RETURN (Thr.Count>1) & (~TERMINATION.HasHalted()) & SEM.CondClaim(s)
  END CondClaim;

PROCEDURE IsSemaphoreException * (): BOOLEAN;
  BEGIN
    RETURN SEM.IsSemaphoreException()
  END IsSemaphoreException;

END mthrSemaphores.