(** This module contains some routins concerned with thrSym module.

    Last changed: 27.03.2015 *)

MODULE mthrUtils;

IMPORT
  Thr:=mthrSym, HU:=mhostUtils, Win:=Windows;

TYPE
  Procedure * = PROCEDURE;

  SimpleProc = RECORD (Thr.Proc)
    p:Procedure;
  END;
  (* This procedure executes a simple procedure in the threads. *)

PROCEDURE (VAR p:SimpleProc) Do ();
  BEGIN
    ASSERT(p.p#NIL);

    p.p()
  END Do;

PROCEDURE ExecProc * (p:Procedure);
  (** This procedure executes p in all the threads. *)
  VAR
    sp:POINTER TO SimpleProc;
  BEGIN
    NEW(sp);
    sp.p:=p;
    Thr.Exec(sp);
  END ExecProc;

PROCEDURE LowestThreadPriority();
  BEGIN
    HU.SetLowestPriority();
  END LowestThreadPriority;

PROCEDURE HighestThreadPriority();
  BEGIN
    HU.SetHighestPriority();
  END HighestThreadPriority;

PROCEDURE SetLowestPriority * ();
  (** Sets lowest priority of all threads in SymThreads module *)
  BEGIN
    ASSERT(~Thr.executing);

    ExecProc(LowestThreadPriority);
  END SetLowestPriority;

PROCEDURE SetHighestPriority * ();
  (** Sets highest priority of all threads in SymThreads module *)
  BEGIN
    ASSERT(~Thr.executing);

    ExecProc(HighestThreadPriority);
  END SetHighestPriority;

PROCEDURE Inc * (VAR i:LONGINT): LONGINT;
  (** Increments the i by 1 and return the result. Interlocked operation. *)
  BEGIN
    <* PUSH *>
    <* WOFF311 + *>
    IF Thr.Count<=1 THEN
      INC(i);
      RETURN i;
    ELSE
      RETURN Win.InterlockedIncrement(i)
    END;
    <* POP *>
  END Inc;

END mthrUtils.