(** This module contains geometry of the simulations and all the parameters,
    which are specific for this geometry. *)

<* O2EXTENSIONS + *>
MODULE Geometry;

IMPORT
  Geo:=SHIPGeometry, Math:=mMath, Protocol, COMPILER, PHC:=msciPhConst;

CONST
  Frequency * = Geo.Frequency; (** RF frequency in MHz *)

  CoilZ1 * = Geo.CoilZ1; (** z of the center of the central turn *)

  (** Gabarit sizes of geometry and plasma *)
  TotalMinZ * = Geo.TotalMinZ;
  TotalMaxZ * = Geo.TotalMaxZ;
  TotalRad * = Geo.TotalRad;

  (** Plasma is calculated in this region: *)
  PlasmaMinZ * = Geo.PlasmaMinZ;
  PlasmaMaxZ * = Geo.PlasmaMaxZ;
  PlasmaRad * = Geo.PlasmaRad;

  CoilCrossSectionArea * = Geo.CoilCrossSectionArea; (** Area of cross-section of the coil *)

  ModuleName = COMPILER.EQUATION('module');
  cm = PHC.cm;

TYPE
  Float = Math.Float;
  SpaceType * = Geo.SpaceType; (** Type of spaces in the model *)

CONST
  (** Items of Type *)
  In * = Geo.In; (** In is a free space inside the torch (filled by Ar) *)
  Out * = Geo.Out; (** Out is a free space outside the torch (filled by air) *)
  Glass * = Geo.Glass; (** Glass is a space inside glass or alumna walls *)
  Coil * = Geo.Coil; (** Coil is a space inside the coil *)

PROCEDURE GetSpaceType * (z,r:Float): SpaceType;
  (** Returns type of the given point.
      (z,r) may exceed dimensions of the cell. *)
  BEGIN
    RETURN Geo.GetSpaceType(z,r)
  END GetSpaceType;

PROCEDURE InitialElectricalConductivity * (z,r:Float): Float;
  (** Electrical conductivity at the 1st iteration *)
  BEGIN
    RETURN Geo.InitialElectricalConductivity(z,r)
  END InitialElectricalConductivity;

BEGIN
  Protocol.String('MODULE ',ModuleName,':');
  Protocol.Real('CoilZ1 = ',CoilZ1/cm,' cm');
  Protocol.Real('Frequency = ',Frequency,' MHz');
  Protocol.Real('TotalMinZ = ',TotalMinZ/cm,' cm');
  Protocol.Real('TotalMaxZ = ',TotalMaxZ/cm,' cm');
  Protocol.Real('TotalRad = ',TotalRad/cm,' cm');
  Protocol.Real('PlasmaMinZ = ',PlasmaMinZ/cm,' cm');
  Protocol.Real('PlasmaMaxZ = ',PlasmaMaxZ/cm,' cm');
  Protocol.Real('PlasmaRad = ',PlasmaRad/cm,' cm');
  Protocol.String('','','');
END Geometry.