(** This module provides common input-output operations. *)
<* O2EXTENSIONS + *>
MODULE InOut;

IMPORT
  ComFunc:=mCommonFunctions, HU:=mhostUtils, DStr:=msysDynStr, Math:=mMath,
  Area:=mArea2D, AreaC:=mArea2DComplex, ComFuncC:=mCommonFunctionsComplex;

TYPE
  Float = Math.Float;
  String = DStr.String;

VAR
  IterationNumber - : LONGINT; (** Zero-based number of the current iteration *)
  OutDir - :String;            (** Some procedures copy files to this directory additionally *)

PROCEDURE NextIteration * ();
  (** Sets the next iteration: increases IterationNumber, creates
      the corresponding directory and saves its name to OutDir *)
  BEGIN
    INC(IterationNumber);
    OutDir:=DStr.FromInt(IterationNumber);
    HU.MakeDir(OutDir^);
    OutDir:=DStr.Concat2(OutDir^,'\');
  END NextIteration;

<* PUSH *>
<* WOFF903 + *> (* sunction result is not used *)
PROCEDURE CopyFile * (FileName-:ARRAY OF CHAR);
  (** Copies the specified file to the OutDir *)
  VAR
    b:BOOLEAN;
  BEGIN
    b:=HU.CopyFile(FileName,DStr.Concat2(OutDir^,FileName)^,TRUE);
    ASSERT(b);
  END CopyFile;
<* POP *>

PROCEDURE SaveFunc * (a-:Area.Area; VAR f:ComFunc.Float2;
                      FileName-:ARRAY OF CHAR; factor:Float);
  (** Saves the specified function in the points specified by the area,
      with the factor, to the specified file, and copies it additionally
      to OutDir *)
  BEGIN
    ASSERT(Area.Correct(a));
    ASSERT(IterationNumber>0);

    Area.SaveFunc(a,f,FileName,factor);
    CopyFile(FileName);
  END SaveFunc;

PROCEDURE SaveSimpleFunc * (a-:Area.Area; f:ComFunc.Simple2;
                            FileName-:ARRAY OF CHAR; factor:Float);
  (** Saves the specified function in the points specified by the area,
      with the factor, to the specified file, and copies it additionally
      to OutDir *)
  BEGIN
    ASSERT(Area.Correct(a));
    ASSERT(IterationNumber>0);
    ASSERT(f#NIL);

    Area.SaveSimpleFunc(a,f,FileName,factor);
    CopyFile(FileName);
  END SaveSimpleFunc;

PROCEDURE SaveFuncC * (a-:Area.Area; VAR f:ComFuncC.Float2;
                       FileName-:ARRAY OF CHAR; factor:Float);
  (** Saves the specified function in the points specified by the area,
      with the factor, to the specified file, and copies it additionally
      to OutDir *)
  BEGIN
    ASSERT(Area.Correct(a));
    ASSERT(IterationNumber>0);

    AreaC.SaveFunc(a,f,FileName,factor);
    CopyFile(FileName);
  END SaveFuncC;

PROCEDURE SaveSimpleFuncC * (a-:Area.Area; f:ComFuncC.Simple2;
                             FileName-:ARRAY OF CHAR; factor:Float);
  (** Saves the specified function in the points specified by the area,
      with the factor, to the specified file, and copies it additionally
      to OutDir *)
  BEGIN
    ASSERT(Area.Correct(a));
    ASSERT(IterationNumber>0);
    ASSERT(f#NIL);

    AreaC.SaveSimpleFunc(a,f,FileName,factor);
    CopyFile(FileName);
  END SaveSimpleFuncC;

PROCEDURE SaveArray * (a-:Area.Area; field-:Math.Array2;
                       FileName-:ARRAY OF CHAR; factor:Float);
  (** Saves the specified array in the specified area,
      with the factor, to the specified file, and copies it additionally
      to OutDir *)
  BEGIN
    ASSERT(Area.Correct(a));
    ASSERT(IterationNumber>0);

    Area.SaveArray(a,field,FileName,factor);
    CopyFile(FileName);
  END SaveArray;

BEGIN
  IterationNumber:=0;
  OutDir:=NIL;
END InOut.