<* O2EXTENSIONS + *>
<* O2ADDKWD + *>
MODULE Protocol;

IMPORT
  Out:=msysTextOut, DStr:=msysDynStr, ProgEnv, SysClock, HU:=mhostUtils,
  SU:=msysUtils, TERMINATION, Disp:=mhostDisplay, FileSys, Math:=mMath,
  Thr:=mthrSym, SEM:=mthrSemaphores, Vec:=m3DVectors, SOut:=mhostStdOut,
  Com:=Common;

CONST
  ModuleName = 'Protocol';

  Name * = 'Protocol.txt'; (** The protocol file name *)
  LogName * = 'extra.dat'; (** The log file name *)
  ScreenFileName * = 'Screen.dat';

  Tab * = Out.Tab;

TYPE
  Float = Math.Float;

VAR
  LogFile - :Out.Formatter; (** formatter of the log file *)

  allowed * : BOOLEAN;

PROCEDURE String * (prefix-,string-,suffix-:ARRAY OF CHAR);
  (** Writes prefix, then string, then suffix to the protocol file. *)
  VAR
    f:Out.Formatter;
  BEGIN
    IF allowed THEN
      ASSERT(~Thr.executing);

      Out.OpenFile(f,Name,Out.Append);
      Out.String(f,prefix,0);
      Out.String(f,string,0);
      Out.String(f,suffix,0);
      Out.Ln(f);
      Out.Close(f);
    END;
  END String;

PROCEDURE Int * (prefix-:ARRAY OF CHAR; int:LONGLONGINT; suffix-:ARRAY OF CHAR);
  (** Writes prefix, then int, then suffix to the protocol file. *)
  BEGIN
    String(prefix,DStr.FromInt(int)^,suffix);
  END Int;

PROCEDURE Real * (prefix-:ARRAY OF CHAR; x:LONGLONGREAL; suffix-:ARRAY OF CHAR);
  (** Writes prefix, then x, then suffix to the protocol file. *)
  BEGIN
    String(prefix,DStr.FromReal(x)^,suffix);
  END Real;

PROCEDURE Bool * (prefix-:ARRAY OF CHAR; b:BOOLEAN; suffix-:ARRAY OF CHAR);
  (** Writes prefix, then b, then suffix to the protocol file. *)
  BEGIN
    IF b THEN
      String(prefix,'TRUE',suffix);
    ELSE
      String(prefix,'FALSE',suffix);
    END;
  END Bool;

PROCEDURE AndOutString * (prefix-,string-,suffix-:ARRAY OF CHAR);
  (** Protocols the strings and prints then in the screen *)
  BEGIN
    String(prefix,string,suffix);
    SOut.String(prefix,0);
    SOut.String(string,0);
    SOut.String(suffix,0);
    SOut.Ln;
  END AndOutString;

VAR
  LogCriticalSection:SEM.SEMAPHORE;

PROCEDURE LogString * (prefix-,string-,suffix-:ARRAY OF CHAR);
  (** Writes prefix, then string, then suffix to the log file. *)
  BEGIN
    SEM.Claim(LogCriticalSection);
    Out.String(LogFile,prefix,0);
    Out.String(LogFile,string,0);
    Out.String(LogFile,suffix,0);
    Out.Ln(LogFile);
    SEM.Release(LogCriticalSection);
  END LogString;

PROCEDURE LogInt * (prefix-:ARRAY OF CHAR; int:LONGLONGINT; suffix-:ARRAY OF CHAR);
  (** Writes prefix, then int, then suffix to the log file. *)
  BEGIN
    LogString(prefix,DStr.FromInt(int)^,suffix);
  END LogInt;

PROCEDURE LogReal * (prefix-:ARRAY OF CHAR; x:LONGLONGREAL; suffix-:ARRAY OF CHAR);
  (** Writes prefix, then x, then suffix to the log file. *)
  BEGIN
    LogString(prefix,DStr.FromReal(x)^,suffix);
  END LogReal;

PROCEDURE LogVector * (prefix-:ARRAY OF CHAR; v-:Vec.Vector; suffix-:ARRAY OF CHAR);
  VAR
    s:DStr.String;
  BEGIN
    s:=DStr.Concat5(DStr.FromReal(v.x)^,' ',DStr.FromReal(v.y)^,' ',DStr.FromReal(v.z)^);
    LogString(prefix,s^,suffix);
  END LogVector;

PROCEDURE Log2Vectors * (prefix-:ARRAY OF CHAR; v1-,v2-:Vec.Vector; suffix-:ARRAY OF CHAR);
  VAR
    s1:DStr.String;
    s2:DStr.String;
  BEGIN
    s1:=DStr.Concat5(DStr.FromReal(v1.x)^,' ',DStr.FromReal(v1.y)^,' ',DStr.FromReal(v1.z)^);
    s2:=DStr.Concat5(DStr.FromReal(v2.x)^,' ',DStr.FromReal(v2.y)^,' ',DStr.FromReal(v2.z)^);
    LogString(prefix,DStr.Concat3(s1^,'   ',s2^)^,suffix);
  END Log2Vectors;

PROCEDURE WriteScreen;
  VAR
    ScreenSize:Disp.COORD;
    f:Out.Formatter;
    i:LONGINT;
  BEGIN
    ScreenSize:=Disp.Size();
    Out.OpenFile(f,ScreenFileName,Out.Rewrite);
    FOR i:=0 TO ScreenSize.y-1 DO
      Out.DString(f,Disp.Read(0,i,ScreenSize.x)^,0);
      Out.Ln(f);
    END;
    Out.Close(f);
  EXCEPT
    Out.Close(f)
  END WriteScreen;

PROCEDURE DateTime ();
  VAR
    DT:SysClock.DateTime;
    day,month,year,hour,minute,second:LONGINT;
    s:DStr.String;
  BEGIN
    SysClock.GetClock(DT);

    day:=DT.day;
    month:=DT.month;
    year:=DT.year;
    hour:=DT.hour;
    minute:=DT.minute;
    second:=DT.second;

    s:=DStr.Concat4(DStr.FromIntEx(day,2,0)^,'.',DStr.FromIntEx(month,2,0)^,'.');
    String('Date: ',s^,DStr.FromInt(year)^);

    s:=DStr.Concat4(DStr.FromIntEx(hour,2,0)^,':',DStr.FromIntEx(minute,2,0)^,':');
    String('Time: ',s^,DStr.FromIntEx(second,2,0)^);
  END DateTime;

PROCEDURE ProgramInfo * ();
  VAR
    s:DStr.String;
  BEGIN
    s:=DStr.New(ProgEnv.ProgramNameLength());
    ProgEnv.ProgramName(s^);
    String('Program name: ',s^,'');
    DateTime();
    String('','','');
  END ProgramInfo;

VAR
  t:LONGINT;
  b:BOOLEAN;

BEGIN
  allowed:=Com.FirstRun(); (* To avoid protocolling of the modules data at more runs *)

  SEM.Create(LogCriticalSection,1);
  FileSys.Remove(LogName,b);

  Out.OpenFile(LogFile,LogName,Out.Rewrite);

  ProgramInfo;

  t:=HU.Ms();
FINALLY
  IF TERMINATION.HasHalted() THEN
    String('Unhandled exception. The screen is written in file '+ScreenFileName,'','');
    WriteScreen;
  END;
  t:=HU.Ms()-t;
  String('','','');
  String('Calculation time: ',SU.MsToStr(t,'.')^,'');
  String('','','');
  String('','','');
  Out.Close(LogFile);
  SEM.Destroy(LogCriticalSection);
END Protocol.